/* dsge_batch_analysis.g
**
** This file runs a batch estimation by running 
**
*/
new;
closeall;
cls;

format /mat /on /mb1 /ros 16,8;
outwidth 256;

/** set seed **/
now = time; 
seed = sumc(now*1000+fmod(now[4], 100));
rndseed seed;

#include dsge_lh.src
#include dsge_mod.src
#include dsge_para.src
#include dsge_procs.src
#include dsge_spec.g;
#include pathspec.g;
#include filenames.g;

/* Step 1: Generate draws from the posterior based on Rsample
*/
step1:
Tend       = Tendvec[1];
datasetstr = filenamestr[1];
labordatasetstr = laborfilenamestr[1];
hmaxact    = hmaxactvec[1];
#include dsge_loaddata.g;

YY    = YYR[Tbegin:Tend,.];
opath = Rpostpath;
//#include dsge_pm.g;

YY    = YYR[Tbegin:Tend,.];
opath = Rpostpath;
lpath = Rpostpath;
//#include dsge_hess.g;

YY      = YYR[Tbegin:Tend,.];
pmsel   = "";
nblocks = 25;
nsim    = 10000;
cc0     = 1;
cc      = 0.2;
opath   = Rparapath;
lpath   = Rpostpath;
mhrun   = "1";

delete_files = 0;
//#include dsge_mh.g;

/* Step 2: Generate hypothetical trajectories
   */
step2:

      /* Simulated data */
      YY      = YYR[Tbegin:Tend,.];
nblock  = 10000;
nuse    = 500;
lpath   = Rparapath;
mhrun   = "1";
Psize   = rows(hmaxactvec)+hmax;
"Psize  ";; Psize;
//#include dsge_gentraj.g;
//#include dsge_extendtraj.g;


/* Step 3: for each trajectory, compute recursive forecasts
   */
step3:
ntraj = 500;
trajind = 0;
do until trajind > ntraj;

   if trajind > 0;
      trajstr = "traj" $+ ftos(trajind,"%*.*lf",1,0);
   else;
      trajstr = "traj0";
   endif;

   /* along each trajectory, there are rows(Tendvec)
      */
   sampleind = 500;//33;
   do until sampleind > rows(Tendvec);

      Tend    = Tendvec[sampleind];
      hmaxact = hmaxactvec[sampleind];

      if trajind > 0;
         lsimdata = specpath $+ "/" $+ trajstr $+ "/YYextend.dat";
         open fhsimdata = ^lsimdata for read;
         YYalltr = readr(fhsimdata,Tend+hmaxact);
         tialltr = seqa(1964.25,0.25,Tend+hmaxact);
         closeall fhsimdata;
         YYR = YYalltr[1:Tend,.];
         tiR = tialltr[1:Tend,.];
         YYP = YYalltr[Tend+1:Tend+hmaxact,.];
         tiP = tialltr[Tend+1:Tend+hmaxact,.];
         //((tiR~YYR)|(tiP~YYP));
         "Loading simulated data set";
      else;
         datasetstr = filenamestr[sampleind];
         labordatasetstr = laborfilenamestr[sampleind];
         #include dsge_loaddata.g;
      endif;
      "Estimation: " tiR[Tbegin] "to " tiR[Tend];
      "Prediction: " tiP[1] "to " tiP[rows(tiP)];

      Tendstr = "Tend" $+ ftos(Tend,"%*.*lf",1,0);
      trajTendpath = specpath $+ "/" $+ trajstr $+ "/" $+ Tendstr;
      chkdir  = chk_dir(trajTendpath);

      /* Bayesian estimation  */
      /************************/

      /* set directory */
      if sampleind == 1;
         lpath = Rparapath $+ "/mhrun1";
         opath = trajTendpath $+ "/para";
      elseif sampleind == 33;
         lpath = specpath $+ "/" $+ trajstr $+ "/" 
               $+ "Tend" $+ ftos(Tend,"%*.*lf",1,0) $+ "/para";
         opath = trajTendpath $+ "/para";
      else;
         lpath = specpath $+ "/" $+ trajstr $+ "/" 
               $+ "Tend" $+ ftos(Tend-1,"%*.*lf",1,0) $+ "/para";
         opath = trajTendpath $+ "/para";
      endif;
      chkdir = chk_dir(opath);

      if sampleind == 1;
         /* for sampleind == 1, just copy the posterior draws from Rparapath
            */
         nburn     = 1;
         blocksize = 250000;
         //#include dsge_postrecursive0.g;
      elseif sampleind == 33;
         /* We have to restimate the model, since we didn't save the 
            ** draws from before because of space considerations...
            */
         YY        = YYR[Tbegin:Tend,.];
         //#include dsge_pm.g;
         //#include dsge_hess.g;
         //parapostmean = para;
         //parapostcov = invHHM;
         nblocks = 25;
         if trajind == 0;
            nsim = 10000;
         else;
            nsim = 1000;
         endif;
         cc0     = 0;
         cc      = 0.2;
         //#include dsge_postrecursive12.g;
      else;
         /* for sampleind > 1, construct and IS estimate of posterior
            ** mean and covariance matrix 
            */
         blocksize = 25000;
         nburn     = 1;
         nuse      = 10;
         YY        = YYR[Tbegin:Tend,.];
         //#include dsge_postrecursive11.g;
         /* Run MH starting from post mean, using post cov as proposal
            */
         nblocks = 25;
         if trajind == 0;
            nsim = 10000;
         else;
            nsim = 1000;
         endif;
         cc0     = 0;
         cc      = 0.2;
         YY      = YYR[Tbegin:Tend,.];
         //#include dsge_postrecursive12.g;
      endif;

      /* Simulate from predictive density  */
      /*************************************/

      nblock = 10000;
      nuse   = 10;
      npaths = 10;
      
      lpath  = trajTendpath $+ "/para"; 
      opath  = trajTendpath $+ "/results"; 


      chkdir=chk_dir(opath);

      YYplus = YYp;
      tiplus = tip;

      YY = YYR[Tbegin:Tend,.];

      //#include fig_irf.g;
      //#include dsge_vardecomp.g;
      //#include dsge_pred.g;      
      //#include dsge_predfast.g;

      /* Evaluate the predictions  */
      /*****************************/
      prun     = "";    // bandwith scale 
      nblock   = 10000;  // block size
      lpath    = trajTendpath $+ "\\results"; 
      opath    = trajTendpath $+ "\\results"; 
      chkdir   = chk_dir (opath);

      YYplus = YYp;
      tiplus = tip;

      // Start with PITs and MSEs
      //#include dsge_predeval.g
      // #include dsge_correval.g;
      // #include dsge_plotjoint.g;

      // Continue with Event Probabilities
      // Some events are defined relative to the 
      // last observation in the sample (predeval2.g)
      //
      // We also consider events that are
      // defined relative to steady-state related
      // parameters (predeval3.g)  

      YYlast = YYR[rows(YYR),.];
      //#include dsge_predeval2.g


      // we are using nblock
      // from above
      nburn = 101;
      lparapath   = trajTendpath $+ "/para"; 
      //#include dsge_predeval3.g
               
      sampleind = sampleind+1;

      endo;

      /* Step 4a: summarize the PITs */

      disp_h = 1|4|8;
      mcdir  = "results";
      prun   = "";
      lpath  = specpath $+ "/" $+ trajstr;
      opath  = specpath $+ "/" $+ trajstr;

      //#include dsge_pitmsesummary.g

      /* Step 4b: summarize event forecasts */
      events_n = 4;
      //#include dsge_eventsummary2.g
      #include dsge_eventsummary3.g

      sampleind = 1;

      if delete_files == 1;
         do until sampleind > rows(Tendvec);
            /* remove unwanted samples */
            Tend    = Tendvec[sampleind];

            lparapath = specpath $+ "/" $+ trajstr $+ "/"
                  $+ "Tend" $+ ftos(Tend,"%*.*lf",1,0) $+ "/para";
            lpredpath = specpath $+ "/" $+ trajstr $+ "/"
                  $+ "Tend" $+ ftos(Tend,"%*.*lf",1,0) $+ "/results/predvar.dat";

            /* zip files -- this is only to save space */
            /*********************************************/
            cmd =  "cd " $+ lparapath $+ " && rm *.dat"; 
            dos ^cmd;
            cmd = "rm " $+ lpredpath;
            dos ^cmd;
            sampleind = sampleind + 1;
         endo;
      endif;
      trajind = trajind+1;

endo;
