/* dsge_eventsummary.g
** This file summarizes the event forecasts
*/

/********************************************************************
* Create log file
********************************************************************/

outstr = opath $+ "\\eventsummary3" $+ ".txt"; 
output file = ^outstr reset;

/********************************************************************
* Load Event probabilities and prediction errors
********************************************************************/

event_prob     = zeros(rows(Tendvec), events_n*hmax);
event_actprob  = zeros(rows(Tendvec), events_n*hmax);
TImat          = zeros(rows(Tendvec), hmax);

ind0 = 1;//33;
ind1 = rows(Tendvec);//50;
sampleind = 1;
do until sampleind > rows(Tendvec);

    Tend         = Tendvec[sampleind];
    Tendstr      = "Tend" $+ ftos(Tend,"%*.*lf",1,0);
    trajTendpath = opath $+ "\\" $+ Tendstr;

    lprob    = trajTendpath $+ "\\" $+ mcdir $+ "\\prob3.dat";
    lactprob = trajTendpath $+ "\\" $+ mcdir $+ "\\actprob3.dat";

    open fhprob    = ^lprob for read;
    open fhactprob = ^lactprob for read;
   
    e_prob     = readr(fhprob,hmax);
    e_actprob  = readr(fhactprob,hmax);
    tiplus = e_prob[.,1];

    e_prob    = e_prob[.,2:cols(e_prob)];
    e_actprob = e_actprob[.,2:cols(e_actprob)];

    event_prob[sampleind,.]    = vec(e_prob')';
    event_actprob[sampleind,.] = vec(e_actprob')';
    TImat[sampleind,.] = tiplus';
   
    closeall fhprob, fhactprob;
   
    sampleind = sampleind + 1;
   
endo;

event_prob = event_prob[ind0:ind1,.];
event_actprob = event_actprob[ind0:ind1,.];
TImat = TImat[ind0:ind1, .];

/********************************************************************
* Summarize event probabilities, careful with missing values
********************************************************************/

e_prob_mean    = meancmiss(event_prob)';
e_actprob_mean = meancmiss(event_actprob)';
nhorizon_disp = rows(disp_h);

"--------------------------------------------------------------";
" Event probabilities";
"--------------------------------------------------------------";
horizonind = 1;
do until horizonind > nhorizon_disp;

   h = disp_h[horizonind];
   h;; "-step ahead forecast";
   e_prob_mean_h = e_prob_mean[.,(h-1)*events_n+1:h*events_n];
   "Average Event Probabilities:" e_prob_mean_h;
   "";
   horizonind = horizonind + 1;
endo;

"--------------------------------------------------------------";
" Average Prediction Errors";
"--------------------------------------------------------------";
horizonind = 1;
do until horizonind > nhorizon_disp;

   h = disp_h[horizonind];
   h;; "-step ahead forecast";
   e_actprob_mean_h = e_actprob_mean[.,(h-1)*events_n+1:h*events_n];
   "Average Prediction Errors:" e_actprob_mean_h;
   "";
   horizonind = horizonind + 1;
endo;

output off;

/*******************************************************************
* Store results
*******************************************************************/

oprob     = opath $+ "\\eventprob3.dat";
oactprob  = opath $+ "\\eventactprob3.dat";

create fhprob    = ^oprob with PROB, hmax*events_n, 8;
create fhactprob = ^oactprob with ACTPROB, hmax*events_n, 8;

wr = writer(fhprob, e_prob_mean);
wr = writer(fhactprob, e_actprob_mean);

closeall fhprob, fhactprob;
