/******************************************************************************
* DSGE_PM.G
* maximizes posterior density
*
* upadted:  06/02/2005
******************************************************************************/

/** Global defaults **/
__output = 1;       /** 1 for verbose of CSMIN **/
__graph  = 0;       /** 1 for graph mode **/
_verbose = 0;       /** 0 for silence, 1 for compact, 2 for detailed **/
_fcmptol = 1E-10;

/********************************************************
*  Import prior specification
********************************************************/
priorfile = specname $+ ".txt";
load path = ^priopath prior[] = ^priorfile;

prior = reshape(prior,rows(prior)/5,5);

{prior,para_names} = psel(prior,msel);

pshape   = prior[.,1];
pmean    = prior[.,2];
pstdd    = prior[.,3];
pmask    = prior[.,4];
pfix     = prior[.,5];
pmaskinv = 1-pmask;
pshape   = pshape.*pmaskinv;

para  = pmsv(msel); 
npara = rows(para);
para  = para.*pmaskinv + pfix.*pmask;

/** parameter specification: transformation **/
_trspec = prtr(msel);
_trspec[.,1] = _trspec[.,1].*pmaskinv;

/** start log file **/
outstr = opath $+ "/" $+ ftos(timedt,"%*.*lf",1,0) $+ "pm.log";
output file = ^outstr reset;

/********************************************************
* Calculate posterior density at starting values
********************************************************/

"Model              " $lmodel;
"Prior              " $lprior;
"Dataset            " $dataselstr;
"Subsample          " $subTstr;
"===================================";
"";
"Prior Density at Starting Value:";; priodens(para,pmean,pstdd,pshape);
lnpy = -objfcn_pm(invtrans(para));
"Posterior at Starting Value:    ";; lnpy;
"";

/********************************************************
* Maximize the posterior density
********************************************************/

/** initial values: x0 and H0 **/
cc = 0.1;
x0 = invtrans(para) + cc* ( rndn(npara,1) );
H0 = eye(npara)*1E-4;

/** set optimization routine environment **/
nit  = 1000;        /** maximum number of iteration **/
crit = 1E-8;        /** tolerance **/

/** Use personal optimization procedure **/
/** translated from Chris Sims' CSMIN package **/
{fh,xh,g,H,itct,fcount,_csretcode} = csminwel(x0,H0,crit,nit,&objfcn_pm);

/** clear screen **/
if __output;
    cls;
else;
    "";
endif;

/********************************************************
* Save parameter estimates and return code
********************************************************/

/** max to model parameters **/
paraest = trans(xh);
paraest = real(paraest);
paraest = paraest.*pmaskinv + pfix.*pmask;


/** output file name **/
oparaest = opath $+ "/pm";
opostval = opath $+ "/pmval";
oretcode = opath $+ "/pmret";

/** make output file **/
create fhparaest = ^oparaest with ^para_names, npara, 8;
wr = writer(fhparaest,paraest');
closeall fhparaest;

create fhretcode = ^oretcode with RETCODE, 1, 2;
wr = writer(fhretcode,_csretcode);
closeall fhretcode;

/********************************************************
* Estimation Output
********************************************************/
{lnpY,retcode,obserror,obsvar} = evalmod(paraest,YY,msel);
lnprio  = priodens( paraest, pmean, pstdd, pshape);

/** posterior values **/
postnames = { LNPOST LNPY LNPRIO };
create fhpostval = ^opostval with ^postnames, 3, 8;
wr = writer(fhpostval,(lnpy+lnprio)~lnpy~lnprio);
closeall fhparaest;

/* Compute gradient in terms of model parameters */
gradmod = g./ diag(gradp(&invtrans,paraest));

"";
"Model              " $lmodel;
"Prior              " $lprior;
"Dataset            " $dataselstr;
"Subsample          " $subTstr;
"===================================";
"Posterior Mode     " (lnpy+lnprio);
"Likelihood at Mode " lnpy;
"Prior at Mode      " lnprio;
"";
"Parameter | Estimate | Start-V | Prior Mean | Prior Stdd | Gradient ";
outmat = para_names'~paraest~trans(x0)~pmean~pstdd~gradmod;
let mask[1,6] = 0 1 1 1 1 1;
let fmt[6,3] = "-*.*s" 10 6
               "*.*lf"  9 4
               "*.*lf" 11 4
               "*.*lf" 11 4
               "*.*lf" 13 4
               "*.*lf" 12 4;
d = printfm(outmat,mask,fmt);
"";
"Return code:        ";; _csretcode;
"";

/** close log file **/
output file = ^outstr off;

