/******************************************************************************
* DSGE_POSTRECURSIVE12.G
* generates draws from the posterior distribution
* via the random walk Metropolis-Hastings algorithm
*
* upadted:  06/16/2005 
******************************************************************************/

/** set seed **/
/** rndseed 1234; **/


/** start log file **/
outstr = opath $+ "\\" $+ ftos(timedt,"%*.*lf",1,0) $+ "postrecursive12.log";
output file = ^outstr reset;

/********************************************************
*  Import prior specification
********************************************************/
priorfile = specname $+ ".txt";
load path = ^priopath prior[] = ^priorfile;
prior = reshape(prior,rows(prior)/5,5);

{prior,para_names} = psel(prior,msel);

pshape   = prior[.,1];
pmean    = prior[.,2];
pstdd    = prior[.,3];
pmask    = prior[.,4];
pfix     = prior[.,5];
pmaskinv = 1-pmask;
pshape   = pshape.*pmaskinv;

/** parameter specification: transformation and bounds **/
_trspec      = prtr(msel);
_trspec[.,1] = _trspec[.,1].*pmaskinv;
_bounds      = _trspec[.,2 3];

/********************************************************
*  Configure the  MCMC
********************************************************/

para = parapostmean;

/********************************************************
* Covariance matrix of proposal distribution
********************************************************/

/** rank of covariance matrix (theoretical) **/
SIGPROPdim = sumc(pmaskinv);

/** log determinant for nonsingular part of post cov matrix **/
{u , s, v} = svd1(parapostcov);
SIGPROPlndet = 0;
sinv = zeros(rows(s),cols(s));
i = 1;
do until i > npara;
   if i <= SIGPROPdim;
      SIGPROPlndet = SIGPROPlndet + ln(s[i,i]);
      sinv[i,i] = 1/s[i,i];
   endif;
   i = i+1;
endo;

/** square root of covariance matrix **/
SIGSCALE = u*sqrt(s);

/** inverse of covariance matrix **/
SIGPROPinv = u*sinv*u';


/********************************************************
* Initialize Output files
********************************************************/

/** print specification **/

"";
"===================================";
"Model            " $lmodel;
"Data set         " $dataselstr;
"Prior            " $lprior;
"Subsample        " $subTstr;
"MHrun Index      " $mhrun;
"Scaling of Hess  " cc;
"";
"===================================";
"";

"";
"Parameter   | Estimate ";
outmat = para_names'~para;
let mask[1,2] = 0 1;
let fmt[2,3]  =
                "-*.*s " 10 6
                "*.*lf " 10 4;
d = printfm(outmat,mask,fmt);
{lnpost0, lnpy0} = objfcn_mh(para);

"";
"Prior*Likelihood at Mode:" lnpost0;
"Likelihood at Mode      :" lnpy0;
"";

statname = { POST LIKE PROP PROPPOST REJECT };

ostat = opath $+ "\\st";
opara = opath $+ "\\pa";
oalpha = opath $+ "\\alpha";
      
create fhstat = ^ostat with ^statname, 5, 4;
create fhpara = ^opara with ^para_names, npara, 4;
create fhalpha = ^oalpha with ALPHA, 8, 4;
/********************************************************
* Metropolis-Hastings algorithm
********************************************************/

/** initialize PARANEW **/
/** trembles the posterior mode **/
valid = 0;
do until valid;
   paranew = para + cc0*(sigscale*rndn(npara,1));
   paranew = paranew.*pmaskinv + pfix.*pmask;

   {postnew, likenew, At} = objfcn_mhfast(paranew);
   propdens  = -0.5*SIGPROPDIM*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*SIGPROPDIM*ln(cc0^2)
               -0.5*(paranew - para)'*SIGPROPINV*(paranew - para)/cc0^2;
   if postnew > -10000;
      valid = 1;
   endif;
endo;

/** block loop **/
iblock = 1;
do until iblock > nblocks;
    
   "";
   "----------------------------------------";
   "Block" iblock "of" nblocks;
   pause(1e-20);

   /** set start time **/
   tstart = date;

   /** storages **/
   parasim = zeros(nsim,npara);
   alphasim = zeros(nsim, rows(At));
   likesim = zeros(nsim,1);
   postsim = zeros(nsim,1);
   rej     = zeros(nsim,1);
   propsim = zeros(nsim,1);
   proppostsim = zeros(nsim,1);

   if (iblock == 1);
      /** save initial value as 1st draw in 1st block **/
      j = 2;
      likesim[1,1] = likenew;
      postsim[1,1] = postnew;
      propsim[1,1] = propdens;
      proppostsim[1,1] = postnew;
      parasim[1,.] = paranew';
      alphasim[1, .] = At';
      postold      = postnew;
      likeold      = likenew;
      paraold      = paranew;
      Atold        = At;
   else;
      /** don't save initial values **/
      j=1;
   endif;

   /** with-in block loop **/
    do until j > nsim;

      /** propose **/
      paranew = paraold + cc*(sigscale*rndn(npara,1));
      paranew = paranew.*pmaskinv + pfix.*pmask;

      /** evaluate at the candidate **/
      {postnew, likenew, Atnew} = objfcn_mhfast(paranew);
      propdens = -0.5*SIGPROPDIM*ln(2*pi) - 0.5*SIGPROPlndet - 0.5*SIGPROPDIM*ln(cc^2)
                 -0.5*(paranew - paraold)'*SIGPROPINV*(paranew - paraold)/cc^2;
      propsim[j,1] = propdens;
      proppostsim[j,1] = postnew;

      /** accept/reject rule  **/
      r = minc(1 | exp( postnew - postold));

      if rndu(1,1) < r;

          /** accept proposed jump **/
          postsim[j,1] = postnew;
          likesim[j,1] = likenew;
          parasim[j,.] = paranew';
          alphasim[j, .] = Atnew';
          /** updates **/
          paraold = paranew;
          postold = postnew;
          likeold = likenew;
          Atold = Atnew;
      else;

          /** reject proposed jump **/
          likesim[j,1] = likeold;
          postsim[j,1] = postold;
          parasim[j,.] = paraold';
          alphasim[j, .] = Atold';
          rej[j] = 1;

      endif;

      j = j+1;
    endo;               /** with-in block  **/

    " ";
    "Rejection: perct " meanc(rej);
    "Likelihood       " meanc(likesim);
    "Posterior        " meanc(postsim);
    "Parameters       "; meanc(parasim)';
    "";

   /** set finish time **/
   tfin = date;

   "Elapsed Time"; etstr(ethsec(tstart,tfin));
   "";

   /** save output **/
   wr = writer(fhstat,postsim~likesim~propsim~proppostsim~rej);
   wr = writer(fhpara,parasim);
   wr = writer(fhalpha,alphasim);
   iblock = iblock+1;

endo;           /** block **/

closeall fhstat, fhpara, fhalpha;

/** close log file **/
output file = ^outstr off;



