/* dsge_predeval.g
**
** Takes draws from predictive density and constructs rmse forecast errors
** for unconditional and conditional forecasts.
*/ 

/*******************************************************************
* Read Forecast Densities
*******************************************************************/

lpred       = lpath $+ "\\predvar.dat";
open fhpred = ^lpred for read;

eofloop = 0;
loopct  = 1;
predsim_total = ones(1,nv_*hmaxact);

"READING FORECAST DENSITIES";

do until eofloop; 

   /** Read blocks of size nblock **/
   predsim = readr(fhpred,nblock);
   predsim_total = predsim_total|predsim;
   loopct = loopct + 1;
	
   eofloop = eof(fhpred);
endo;

predsim_total = predsim_total[2:rows(predsim_total),.];

closeall fhpred;

/*******************************************************************
* Forecast Evaluation
*******************************************************************/

/* PSCALE FIXME */
if prun == "";
   _bscale = 1.0;
elseif prun == "1";
   _bscale = 0.5;
elseif prun == "2";
   _bscale = 0.25;
elseif prun == "3";
   _bscale = 0.10;
else;
   errorlog("Set pscale for prun " $+ prun);
endif;

{pit,mse,bnd} = fcst_eval_np(predsim_total, YYplus, hmaxact,
      average_over_horizon);

/*******************************************************************
* Adjust for missing values
*******************************************************************/

if (hmaxact < hmax);

     pit    = pit | zeros(hmax-hmaxact, cols(pit));
     mse    = mse | zeros(hmax-hmaxact, cols(mse));
     bnd    = bnd | zeros(hmax-hmaxact, cols(bnd));
     tiplus = tiplus | zeros(hmax-hmaxact, 1);
  
     hmaskp =  zeros(hmaxact,cols(pit)) | ones (hmax-hmaxact, cols(pit));
     hmaskm =  zeros(hmaxact,cols(mse)) | ones (hmax-hmaxact, cols(mse));
     hmaskb =  zeros(hmaxact,cols(bnd)) | ones (hmax-hmaxact, cols(bnd));

     pit = missex(pit,hmaskp);
     mse = missex(mse,hmaskm);
     bnd = missex(bnd,hmaskb);
     
endif;

/*******************************************************************
* Write output
*******************************************************************/

opit   = opath $+ "\\pit"  $+ prun $+ ".dat";
obnd   = opath $+ "\\bnd"  $+ prun $+ ".dat";
omse   = opath $+ "\\mse"  $+ prun $+ ".dat";

create fhpit = ^opit with PIT, 1+cols(pit), 8;
create fhbnd = ^obnd with BND, 1+cols(bnd), 8;
create fhmse = ^omse with MSE, 1+cols(mse), 8;

wr = writer(fhpit, tiplus~pit);
wr = writer(fhbnd, tiplus~bnd);
wr = writer(fhmse, tiplus~mse);

closeall fhpit, fhbnd, fhmse;

/*******************************************************************
* Create log
*******************************************************************/

outstr = opath $+ "\\" $+ ftos(timedt,"%*.*lf",1,0) $+ "predeval.log"; 
output file = ^outstr reset;

"MSEs";
tiplus~mse;
"";
"-----------------";
"";
"Probability Integral Transforms";
tiplus~pit;

output off;
"DONE";



