/* vm_pitmsepredcheck.g
**
*/
new;
closeall;
cls;
library pgraph;

format /mat /on /mb1 /ros 10,4;
outwidth 256;

#include dsge_lh.src
#include dsge_mod.src
#include dsge_para.src
#include dsge_procs.src;
#include dsge_spec.g;
#include pathspec.g;

/* Select a forecast horizon
*/
hsel = 1;

/* Load summaries from the actual trajectory
*/
trajstr = "traj0";
lpath   = specpath $+ "\\" $+ trajstr;
prun    = "";
lpitacf = lpath $+ "\\pitacf"      $+ prun $+ ".dat";

open fhpitacf = ^lpitacf for read;
pitacf_act     = readr(fhpitacf,nac_);
pitacf_act     = vec(pitacf_act)';
closeall fhpitacf;

/* Load summaries from the predicted trajectories
*/
ntraj = 500;

pitacf_sim     = zeros(ntraj,cols(pitacf_act));

trajind = 1;
do until trajind > ntraj;

   trajstr = "traj" $+ ftos(trajind,"%*.*lf",1,0);

   lpath   = specpath $+ "\\" $+ trajstr;
   prun    = "";
   lpitacf = lpath $+ "\\pitacf" $+ prun $+ ".dat";

   open fhpitacf = ^lpitacf for read;
   
   pitacf_sim[trajind,.] = vec(readr(fhpitacf,nac_))';

   closeall fhpitacf;

   trajind = trajind+1;

endo;

/* sort columns for simulated statistics
*/

colind = 1;
do until colind > cols(pitacf_sim);
   pitacf_col = pitacf_sim[.,colind];
   pitacf_col = sortc(pitacf_col,1);
   pitacf_sim[.,colind] = pitacf_col;
   colind = colind+1;
endo;

/* construct percentiles for predictive distribution
*/
chkpct_lind = 1+floor(ntraj*chkpct_l);
chkpct_hind = ceil(ntraj*chkpct_h);

pitacf_l     = pitacf_sim[chkpct_lind,.];
pitacf_h     = pitacf_sim[chkpct_hind,.];

/* Arrangement of PIT acfs
**   [Y pi R - Y|Y pi|Y R|Y - Y|pi pi|pi R|pi - Y|R pi|R R|R ] for h1...hmax
** ac1
** ac2
*/

pitacf_l   = reshape(pitacf_l,hmax*nv_*(nv_+1),nac_)';
pitacf_h   = reshape(pitacf_h,hmax*nv_*(nv_+1),nac_)';
pitacf_act = reshape(pitacf_act,hmax*nv_*(nv_+1),nac_)';

// consider only hsel-step ahead forecasts

pitacf_l   = pitacf_l[.,(hsel-1)*nv_*(nv_+1)+1:hsel*nv_*(nv_+1)];
pitacf_h   = pitacf_h[.,(hsel-1)*nv_*(nv_+1)+1:hsel*nv_*(nv_+1)];
pitacf_act = pitacf_act[.,(hsel-1)*nv_*(nv_+1)+1:hsel*nv_*(nv_+1)];

pitacfplot_l   = zeros(nac_,nv_^2);
pitacfplot_h   = zeros(nac_,nv_^2);
pitacfplot_act = zeros(nac_,nv_^2);

/* Create ylabelstr and eliminate x|x forecasts
*/
plotind = 0;
ylabelstr = " ";

varind = 1;
do until varind > nv_;

   condind = 0;
   do until condind > nv_;

      if condind == 0;
         // plot all forecasts
         plotind = plotind+1;
         pitacfplot_l[.,plotind]   = pitacf_l[.,varind];
         pitacfplot_h[.,plotind]   = pitacf_h[.,varind];
         pitacfplot_act[.,plotind] = pitacf_act[.,varind];
         ylabelstr = ylabelstr $| titlestr[varind];
      else;
         // don't plot if varind = condind
         if varind ne condind;
            plotind = plotind+1;
            pitacfplot_l[.,plotind]   = pitacf_l[.,condind*nv_+varind];
            pitacfplot_h[.,plotind]   = pitacf_h[.,condind*nv_+varind];
            pitacfplot_act[.,plotind] = pitacf_act[.,condind*nv_+varind];
            ylabelstr = ylabelstr $| ( titlestr[varind] $+ " | " $+ titlestr[condind] );
         endif;

      endif;

      condind = condind+1;

   endo;

   varind = varind+1;

endo;

ylabelstr = ylabelstr[2:nv_^2+1];

/* Plot PITacf figures
*/

begwind;
window(nv_,nv_,0);

_ptek    = "figure.tkf";
_plwidth = { 10, 10, 10 } ;
_psymsiz = { 10, 10, 10 };
_plctrl  = { 1, 1, 1 };
_pltype  = { 6, 1, 1 };
_pcolor  = { 12, 9, 9 };
_pstype  = { 1, 2, 2};
_pscreen = 1;
_paxht = 0.4;
_pnumht = 0.3;
fonts("microb");

xtics(1,4,1,1);
xgrid=seqa(1,1,nac_);
yticmat = (-1~1~0.5~2) |
          (-1~1~0.5~2) |
          (-1~1~0.5~2) |
          (-1~1~0.5~2) ;



varind = 1;
do until varind > nv_;

   condind = 1;
   do until condind > nv_;

      windind   = (condind-1)*nv_+varind;
      columnind = (varind-1)*nv_+condind;

      ylabelstr[columnind] windind;
      setwind(windind);

      ylabel(ylabelstr[columnind]);
      ytics(yticmat[varind,1],yticmat[varind,2],yticmat[varind,3],yticmat[varind,4]);
      _pline = (1 ~ 6 ~ 0 ~ 0 ~ nac_ ~ 0 ~ 0~ 12 ~ 8);
      xy(xgrid, pitacfplot_act[.,columnind]~pitacfplot_l[.,columnind]~pitacfplot_h[.,columnind]);
      
      condind = condind + 1;
   endo;
     
   varind = varind + 1;
endo;
endwind;
ret = tkf2eps(_ptek,"3eqpitacf.eps");

