/* vm_pitmsepredcheck.g
**
*/
new;
closeall;
cls;
library pgraph;

format /mat /on /mb1 /ros 10,4;
outwidth 256;

#include dsge_lh.src
#include dsge_mod.src
#include dsge_para.src
#include dsge_procs.src;
#include dsge_spec.g;
#include pathspec.g;


/* Load summaries from the actual trajectory
*/
trajstr = "traj0";
lpath   = specpath $+ "\\" $+ trajstr;
prun    = "";
lmse    = lpath $+ "\\mse"      $+ prun $+ ".dat";

open fhmse = ^lmse for read;
MSE_act     = readr(fhmse,1);
closeall fhmse;

/* Load summaries from the predicted trajectories
*/
ntraj = 500;

MSE_sim     = zeros(ntraj,cols(MSE_act));

trajind = 1;
do until trajind > ntraj;

   trajstr = "traj" $+ ftos(trajind,"%*.*lf",1,0);

   lpath   = specpath $+ "\\" $+ trajstr;
   prun    = "";
   lmse    = lpath $+ "\\mse"      $+ prun $+ ".dat";

   open fhmse = ^lmse for read;
   
   MSE_sim[trajind,.]     = readr(fhmse,1);

   closeall fhmse;

   trajind = trajind+1;

endo;

/* sort columns for simulated statistics
*/

colind = 1;
do until colind > cols(MSE_sim);
   MSE_col = MSE_sim[.,colind];
   MSE_col = sortc(MSE_col,1);
   MSE_sim[.,colind] = MSE_col;
   colind = colind+1;
endo;

/* construct percentiles for predictive distribution
*/
chkpct_lind = 1+floor(ntraj*chkpct_l);
chkpct_hind = ceil(ntraj*chkpct_h);

MSE_l     = MSE_sim[chkpct_lind,.];
MSE_h     = MSE_sim[chkpct_hind,.];

/* Rearrange the MSEs
**   Y pi R - Y|Y pi|Y R|Y - Y|pi pi|pi R|pi - Y|R pi|R R|R
** h1
** h2
*/

RMSE_l   = sqrt(reshape(MSE_l,hmax,nv_*(nv_+1)));
RMSE_h   = sqrt(reshape(MSE_h,hmax,nv_*(nv_+1)));
RMSE_act = sqrt(reshape(MSE_act,hmax,nv_*(nv_+1)));

RMSEplot_l   = zeros(hmax,nv_^2);
RMSEplot_h   = zeros(hmax,nv_^2);
RMSEplot_act = zeros(hmax,nv_^2);

/* Create ylabelstr and eliminate x|x forecasts
*/
plotind = 0;
ylabelstr = " ";

varind = 1;
do until varind > nv_;

   condind = 0;
   do until condind > nv_;

      if condind == 0;
         // plot all forecasts
         plotind = plotind+1;
         if varind > 1;
            scale_adj = 4;
         else;
            scale_adj = 1;
         endif;

         RMSEplot_l[.,plotind]   = RMSE_l[.,varind]/scale_adj;
         RMSEplot_h[.,plotind]   = RMSE_h[.,varind]/scale_adj;
         RMSEplot_act[.,plotind] = RMSE_act[.,varind]/scale_adj;
         ylabelstr = ylabelstr $| titlestr[varind];
      else;
         // don't plot if varind = condind
         if varind ne condind;
            plotind = plotind+1;
            RMSEplot_l[.,plotind]   = RMSE_l[.,condind*nv_+varind];
            RMSEplot_h[.,plotind]   = RMSE_h[.,condind*nv_+varind];
            RMSEplot_act[.,plotind] = RMSE_act[.,condind*nv_+varind];
            ylabelstr = ylabelstr $| ( titlestr[varind] $+ " | " $+ titlestr[condind] );
         endif;

      endif;

      condind = condind+1;

   endo;

   varind = varind+1;

endo;

ylabelstr = ylabelstr[2:nv_^2+1];

/* Print some results to the screen
*/
disp_h = 1|2|4;
nfcsts        = nv_*(nv_+1);
nhorizon_disp = rows(disp_h);

"--------------------------------------------------------------";
" Root mean squared errors";
"--------------------------------------------------------------";
horizonind = 1;
do until horizonind > nhorizon_disp;

   h = disp_h[horizonind];
   h;; "-step ahead forecast";
   MSEh = MSE_act[.,(h-1)*nfcsts+1:h*nfcsts];
   
   z = seqa(1,nv_,nv_+1);
   j = 1;
   do until j > nv_;
      MSEjh  = MSEh[.,z];
      RMSEjh = sqrt(MSEjh);
      titlestr[j,.];; RMSEjh;
      z = z+1;
      j = j+1;
   endo;
   "";
   horizonind = horizonind + 1;
endo;

/* Plot RMSE figures
*/

begwind;
window(nv_,nv_,0);

_ptek    = "figure.tkf";
_plwidth = { 10, 10, 10 } ;
_psymsiz = { 10, 10, 10 };
_plctrl  = { 1, 1, 1 };
_pltype  = { 6, 1, 1 };
_pstype  = { 1, 2, 2};

_pcolor  = { 12, 9, 9 };
_pscreen = 1;
_paxht = 0.4;
_pnumht = 0.3;
fonts("microb");

xtics(1,8,1,1);
xgrid=seqa(1,1,hmax);
yticmat = (0~1~0.25~2) |
          (0~0.5~0.1~2)     |
          (0~0.5~0.1~2);




varind = 1;
do until varind > nv_;

   condind = 1;
   do until condind > nv_;

      windind   = (condind-1)*nv_+varind;
      columnind = (varind-1)*nv_+condind;

      ylabelstr[columnind] windind;
      setwind(windind);

      ylabel(ylabelstr[columnind]);
      ytics(yticmat[varind,1],yticmat[varind,2],yticmat[varind,3],yticmat[varind,4]);
      
      xy(xgrid, RMSEplot_act[.,columnind]~RMSEplot_l[.,columnind]~RMSEplot_h[.,columnind]);
      
      condind = condind + 1;
   endo;
     
   varind = varind + 1;
endo;
endwind;
ret = tkf2eps(_ptek,"3eqrmse.eps");

