/* dsge_pitsummary.g
**
** This file plots the empirical CDFs probability integral
** transforms, results of KS tests, and RMSE ratios of conditional vs.
** unconditional forecasts.
**
*/

/********************************************************************
* Create log file
********************************************************************/

outstr = opath $+ "\\pitmsesummary" $+ prun $+ ".txt"; 
output file = ^outstr reset;

/********************************************************************
* Load MSEs and PITs
********************************************************************/
npit = rows(Tendvec);
PIT     = zeros(npit, nv_*(nv_+1)*hmax);
MSE     = zeros(npit, nv_*(nv_+1)*hmax);
TImat   = zeros(npit, hmax);

ind0 = 1;
ind1 = rows(Tendvec);
sampleind = 1;
do until sampleind > rows(Tendvec);

    Tend         = Tendvec[sampleind];
    Tendstr      = "Tend" $+ ftos(Tend,"%*.*lf",1,0);
    trajTendpath = opath $+ "\\" $+ Tendstr;

    lpit  = trajTendpath $+ "\\" $+ mcdir $+ "\\pit" $+ prun $+ ".dat";
    lmse =  trajTendpath $+ "\\" $+ mcdir $+ "\\mse" $+ prun $+ ".dat";

    open fhpit = ^lpit for read;
    open fhmse = ^lmse for read;
   
    p = readr(fhpit,hmax);
    m = readr(fhmse,hmax);
    tiplus = p[.,1];

    p = p[.,2:cols(p)];
    m = m[.,2:cols(m)];
   
    PIT[sampleind,.]   = vec(p')';
    MSE[sampleind,.]   = vec(m')';
    TImat[sampleind,.] = tiplus';

    if sampleind > 39;
        periods_to_kill = sampleind - 39;
        ny = 3;
        x = zeros(1, cols(PIT));
        i0 = cols(x) - (periods_to_kill)*(ny)*(ny+1)+1;
        x[., i0:cols(x)] = ones(1, periods_to_kill*ny*(ny+1));      
        MSE[sampleind, .] = missex(MSE[sampleind, .],x);
        PIT[sampleind, .] = missex(PIT[sampleind, .],x);

    endif;
    closeall fhpit, fhmse;
   
    sampleind = sampleind + 1;
   
endo;
PIT = PIT[ind0:ind1, .];
MSE = MSE[ind0:ind1, .];
TImat = TImat[ind0:ind1, .];
/********************************************************************
* Summarize MSEs, careful with missing values
********************************************************************/

MSEmean = meancmiss(MSE)';

nfcsts        = nv_*(nv_+1);
nhorizon_disp = rows(disp_h);

"--------------------------------------------------------------";
" Root mean squared errors";
"--------------------------------------------------------------";
horizonind = 1;
do until horizonind > nhorizon_disp;

   h = disp_h[horizonind];
   h;; "-step ahead forecast";
   MSEh = MSEmean[.,(h-1)*nfcsts+1:h*nfcsts];
   
   z = seqa(1,nv_,nv_+1);
   j = 1;
   do until j > nv_;
      MSEjh  = MSEh[.,z];
      RMSEjh = sqrt(MSEjh);
      titlestr[j,.];; RMSEjh;
      z = z+1;
      j = j+1;
   endo;
   "";
   horizonind = horizonind + 1;
endo;

/********************************************************************
* Summarize PITs, careful with missing values
********************************************************************/

PITbin  = seqa(1/npitbin_,1/npitbin_,npitbin_);
PITfreq = zeros(npitbin_,cols(PIT));
PITchi2 = zeros(1,cols(PIT));
PITacf  = zeros(nac_,cols(PIT));

colind = 1;
do until colind > cols(PIT);

   PITcol  = packr(PIT[.,colind]);
   PITfreq[.,colind] = 100*counts(PITcol,PITbin)/rows(PITcol);
   PITchi2[1,colind] = sumc( (counts(PITcol,PITbin)-rows(PITcol)/npitbin_)^2./(rows(PITcol)/npitbin_) );
   PITacf[.,colind]  = acf(PITcol,nac_,0);
   colind = colind+1;
   
endo;

"--------------------------------------------------------------";
" Properties of the PITs";
"--------------------------------------------------------------";

horizonind = 1;
do until horizonind > nhorizon_disp;
   
   h = disp_h[horizonind];
   h;; "-step ahead forecast";
   
   "                    Frequencies                       ";

   PITh = PITfreq[.,(h-1)*nfcsts+1:h*nfcsts];
   
   z = seqa(1,nv_,nv_+1);

   j = 1;
   do until j > nv_;
      titlestr[j,.];; PITh[.,z];
      
      z = z+1;
      j = j+1;
	 
   endo;
   
   horizonind = horizonind + 1;
	 
endo;

output off;

/*******************************************************************
* Store results
*******************************************************************/

opit     = opath $+ "\\pitfreq"  $+ prun $+ ".dat";
opitchi2 = opath $+ "\\pitchi2"  $+ prun $+ ".dat";
opitacf  = opath $+ "\\pitacf"   $+ prun $+ ".dat";
omse     = opath $+ "\\mse"      $+ prun $+ ".dat";

create fhpit     = ^opit with PITfreq, rows(PITfreq)*cols(PITfreq), 8;
create fhpitchi2 = ^opitchi2 with PITCHI2, cols(PITchi2), 8;
create fhpitacf  = ^opitacf with PITACF, cols(PITacf), 8;
create fhmse     = ^omse with MSEmean, cols(MSEmean), 8;

wr = writer(fhpit, vec(PITfreq)');
wr = writer(fhpitchi2, PITchi2);
wr = writer(fhpitacf, PITacf);
wr = writer(fhmse, MSEmean);

closeall fhpit, fhmse, fhpitchi2, fhpitacf;

