/* dsge_pred.g
**
** The program computes trajectories from the 
** the DSGE model.
*/

/********************************************************************
* Initialize input files 
********************************************************************/

lpara       = lpath $+ "\\pa.dat";
open fhpara = ^lpara for read;

/********************************************************************
* Initialize output files
********************************************************************/
opred         = opath $+ "\\predvar.dat";
create fhpred = ^opred with PRED, nv_*hmaxact, 8;

/********************************************************************
* Generated draws from predictive density
********************************************************************/

predsim_sum = zeros(1,nv_*hmaxact);
nsim_sum    = 0;

eofloop = 0;

do until eofloop; 

   /** Read blocks of size nblock **/
   parasim = readr (fhpara, nblock);
   nread   = rows (parasim);
   npara   = cols (parasim);

   /** use every nuse-th draw **/
   indseq  = seqa (1, 1, nread);
   indseq  = indseq % nuse;
   indseq  = indseq .ne 0;

   parasim = delif (parasim, indseq);
   nread   = rows (parasim);

   if ismiss (parasim) == 0;

      eofloop = eof (fhpara);   
      j = 1;     
      predsim = zeros(nread*npaths,nv_*hmaxact);
      
      do until j > nread;
    	 predsim[(j-1)*npaths+1:j*npaths,.] 
            = dsgepredmult(parasim[j,.]',YY,hmaxact,npaths);
     	 j = j+1;
      endo;

      predsim_sum = predsim_sum + sumc(predsim)';
      nsim_sum    = nsim_sum + nread*npaths;
      
      // Write Output
      wr = writer (fhpred, predsim);

   endif;

endo;

closeall fhpara, fhpred;

/* Print Posterior Mean Forecasts
*/
outstr = opath $+ "\\" $+ ftos(timedt,"%*.*lf",1,0) $+ "pred.log"; 
output file = ^outstr reset;

predsim_mean= predsim_sum/nsim_sum;
predsim_mean= reshape(predsim_mean, nv_, hmaxact)';

"Predicted:";      
tiplus~predsim_mean;
"Actual:";
tiplus~YYplus;
output off;
      
"";
"DONE";
"";


