/* dsge_predeval3.g
**
** Reads trajectories from predictive density and constructs 
** even probabilities 
*/ 

/*******************************************************************
* Read Forecast Densities
*******************************************************************/

lpred       = lpath $+ "\\predvar.dat";
open fhpred = ^lpred for read;

eofloop = 0;
loopct  = 1;
predsim_total = ones(1,nv_*hmaxact);

"READING FORECAST DENSITIES";

do until eofloop; 

   /** Read blocks of size nblock **/
   predsim = readr(fhpred,nblock);
   predsim_total = predsim_total|predsim;
   loopct = loopct + 1;
	
   eofloop = eof(fhpred);
endo;

predsim_total = predsim_total[2:rows(predsim_total),.];

closeall fhpred;

/*******************************************************************
* Read Parameters to determine cutoffs
*******************************************************************/

lpara       = lparapath $+ "\\pa.dat";
open fhpara = ^lpara for read;

eofloop = 0;
loopct  = 1;
parasum = zeros(1,cols(YYlast));

"READING PARAMETERS";

sr = seekr(fhpara,nburn);
nread_total = 0;

do until eofloop; 

   /** Read blocks of size nblock **/
   parasim = readr(fhpara,nblock);
   nread   = rows(parasim);

   // create target values
   // rr_steady  = para[8];
   // pi_steady  = para[9];
   // gam_steady = para[10];
   //
   // DD[eq_ygr,1] = gam_steady;
   // DD[eq_pi,1]  = pi_steady;
   // DD[eq_r,1]   = pi_steady + rr_steady + 4*gam_steady;
 
   parasim = parasim[.,10]
             ~ parasim[.,9]
             ~ (parasim[.,9]+parasim[.,8]+4*parasim[.,10]);
 
   parasum = parasum + sumc(parasim)';
   nread_total = nread_total + nread;
   eofloop = eof(fhpara);

endo;

ssmean = parasum/nread_total;

closeall fhpara;

/*******************************************************************
* Forecast Evaluation
*******************************************************************/

/* ordering of variables:
** "GDP" $| "INF" $| "FFR";
**
** events_ is ny*events_n
*/

lb_min = -1E8;
lb_max =  1E8;

events_lb = lb_min*ones(cols(YYlast),4);
events_ub = lb_max*ones(cols(YYlast),4);

/* bounds for events are specified
** in terms of specification of events: use
** 1 for ygr vs steady state growth
** 2 for pi vs steady state pi
** 3 for R vs steady state R
*/

/* Event 1: ygr(+), pi(+) */
events_lb[1,1] = ssmean[1];
events_lb[2,1] = ssmean[2];

/* Event 2: ygr(-), pi(-) */
events_ub[1,2] = ssmean[1];
events_ub[2,2] = ssmean[2];

/* Event 3: ygr(-), pi(+) */
events_ub[1,3] = ssmean[1];
events_lb[2,3] = ssmean[2];

/* Event 4: ygr(+), pi(-) */
events_lb[1,4] = ssmean[1];
events_ub[2,4] = ssmean[2];

{events_prob, events_act} 
  = fcst_eval_events(predsim_total, YYplus, hmaxact, events_lb, events_ub, average_over_horizon);

/*******************************************************************
* Adjust for missing values
*******************************************************************/

if (hmaxact < hmax);

     events_prob = events_prob | zeros(hmax-hmaxact, cols(events_prob));
     events_act  = events_act  | zeros(hmax-hmaxact, cols(events_act));
     tiplus = tiplus | zeros(hmax-hmaxact, 1);
  
     hmaskp =  zeros(hmaxact,cols(events_prob)) 
                | ones (hmax-hmaxact, cols(events_prob));
     hmaska =  zeros(hmaxact,cols(events_act)) 
                | ones (hmax-hmaxact, cols(events_act));
     
     events_prob = missex(events_prob,hmaskp);
     events_act  = missex(events_act ,hmaska);
     
endif;

/*******************************************************************
* Write output
*******************************************************************/

oprob      = opath $+ "\\prob3.dat"    ;
oactprob   = opath $+ "\\actprob3.dat" ;

create fhprob    = ^oprob with PROB, 1+cols(events_prob), 8;
create fhactprob = ^oactprob with ACTPROB, 1+cols(events_act), 8;

wr = writer(fhprob,    tiplus[1:rows(events_prob), .]~events_prob);
wr = writer(fhactprob, tiplus[1:rows(events_prob), .]~(events_act-events_prob));

closeall fhprob, fhactprob;

/*******************************************************************
* Create log
*******************************************************************/

outstr = opath $+ "\\" $+ ftos(timedt,"%*.*lf",1,0) $+ "predeval3.log"; 
output file = ^outstr reset;

"Events: lower bounds";
events_lb;

"Events: upper bounds";
events_ub;

"";
"-----------------";
"";

"Predicted Probabilities";
tiplus[1:rows(events_prob), .]~events_prob;
"";
"-----------------";
"";
"Actual Outcomes";
tiplus[1:rows(events_prob), .]~events_act;
"";
"-----------------";
"";
"Prediction Errors";
tiplus[1:rows(events_prob), .]~(events_act-events_prob);


output off;
"DONE";

/* proc(2) fcst_eval_events = (YYpred, YYact, hmax, events_lb, events_ub,average_over_horizon)
**  PURPOSE:
**    Nonparametric probability integral transforms and mean squared
**    errors of forecasts under the unconditional and conditional
**    distributions.
** 
**  NOTES:
**    The conditional distributions are approximated via a (normal) kernel density
**    estimation.  The global variables _bscale scales the bandwidth.
** 
**    For simplicitiy, this only evaluates contemporenous forecasts under the 
**    conditional distributions.
** 
**  INPUTS: 
**    YYpred    - npred x ny*hmax matrix of draws 
**                from predictive distribution
**    YYact     - 1 x ny*hmax vector of actual observation 
**    hmax      - maximum number of steps ahead forecasted
**    events_lb - vector of ny*events_n lower bounds for events
**    events_ub - vector of ny*events_n upper bounds for events
**
**  OUTPUTS:
**   
**    events_prob - hmax*events_n matrix of predicted event probabilities 
**    events_act  - hmax*events_n matrix of actuals
** 
*/
proc(2) = fcst_eval_events(YYpred, YYact, hmax, events_lb, events_ub,average_over_horizon);
   local ny, events_n, events_prob, events_act, event_ind,
         h, hth_sel, YYpredh, events_ind, lb, ub, 
         YYeventind;
   
   ny       = cols(YYact);
   events_n = cols(events_lb);
   
   if (average_over_horizon);
      YYpred = transform_YY_means(YYpred, ny);
      YYact  = vec(YYact)';
      YYact  = transform_YY_means(YYact,ny);
      YYact  = reshape(YYact,ny,hmax)';
   endif;
   
   events_prob = zeros(hmax,events_n);
   events_act  = zeros(hmax,events_n);
   
   h = 1;   
   do until h > hmax;

      // We need to the select the ith step forecast
      hth_sel = seqa(0,hmax,ny)+h;	    
      YYpredh = YYpred[.,hth_sel];

      events_ind = 1;
      do until events_ind > events_n;

         lb = events_lb[.,events_ind]';
         ub = events_ub[.,events_ind]';

         YYeventind = (YYpredh .GE lb) .AND (YYpredh .LE ub);
         YYeventind = prodc(YYeventind');
         events_prob[h,events_ind] = meanc(YYeventind);

         YYeventind = (YYact[h,.] .GE lb) .AND (YYact[h,.] .LE ub);
         YYeventind = prodc(YYeventind');
         events_act[h,events_ind] = YYeventind;
         
         events_ind = events_ind+1;
         
      endo;
               
      h = h+1;
   endo;
   
   retp(events_prob, events_act);

endp;

