/* fig_eventprobpredcheck.g
**
*/
new;
closeall;
cls;
library pgraph;

format /mat /on /mb1 /ros 10,4;
outwidth 256;

#include dsge_lh.src
#include dsge_mod.src
#include dsge_para.src
#include dsge_procs.src;
#include dsge_spec.g;
#include pathspec.g;

events_n = 4;

/* Load summaries from the actual trajectory
*/
trajstr  = "traj0";
lpath    = specpath $+ "\\" $+ trajstr;
lactprob = lpath $+ "\\eventactprob3.dat";

open fhactprob = ^lactprob for read;
actprob_act = readr(fhactprob,1);
closeall fhactprob;

/* Load summaries from the predicted trajectories
*/
ntraj = 500;

actprob_sim     = zeros(ntraj,cols(actprob_act));

trajind = 1;
do until trajind > ntraj;

   trajstr = "traj" $+ ftos(trajind,"%*.*lf",1,0);

   lpath   = specpath $+ "\\" $+ trajstr;
   lactprob = lpath $+ "\\eventactprob3.dat";

   open fhactprob = ^lactprob for read;
   actprob_sim[trajind,.] = readr(fhactprob,1);

   closeall fhactprob;

   trajind = trajind+1;

endo;

/* sort columns for simulated statistics
*/

colind = 1;
do until colind > cols(actprob_sim);
   actprob_col = actprob_sim[.,colind];
   actprob_col = sortc(actprob_col,1);
   actprob_sim[.,colind] = actprob_col;
   colind = colind+1;
endo;

/* construct percentiles for predictive distribution
*/
chkpct_lind = 1+floor(ntraj*chkpct_l);
chkpct_hind = ceil(ntraj*chkpct_h);

actprob_l     = actprob_sim[chkpct_lind,.];
actprob_h     = actprob_sim[chkpct_hind,.];

/* Rearrange the pred errors
**   Event 1   Event 2
** h1
** h2
*/

actprob_l   = reshape(actprob_l,hmax,events_n);
actprob_h   = reshape(actprob_h,hmax,events_n);
actprob_act = reshape(actprob_act,hmax,events_n);

/* Plot prediction errors
*/

begwind;
window(3,2,0);

_ptek    = "figure.tkf";
_plwidth = { 10, 10, 10 } ;
_psymsiz = { 10, 10, 10 };
_plctrl  = { 1, 1, 1 };
_pltype  = { 6, 1, 1 };
_pcolor  = { 12, 9, 9 };
_pstype  = { 1, 2, 2};
_pscreen = 1;
_paxht = 0.3;
_pnumht = 0.3;
_ptitlht = 0.3; 
fonts("microb");

xtics(1,8,1,1);
xgrid=seqa(1,1,hmax);
yticmat = (-0.5~0.5~0.5~2) |
          (-0.5~0.5~0.5~2) |
	      (-0.5~0.5~0.5~2) |
	      (-0.5~0.5~0.5~2) ; 

/* use for eval2 output */
//titlestr =  "GDP(+), INF(+)"
//         $| "GDP(+), INF(-)"
//         $| "GDP(-), INF(+)"
//         $| "GDP(-), INF(-)"
//         ;

/* use for eval3 output */
titlestr =  "GDP(+), INF(+)"
         $| "GDP(-), INF(-)"
         $| "GDP(-), INF(+)"
         $| "GDP(+), INF(-)"
         ;

	  
events_ind = 1;
do until events_ind > 4;
      
      $titlestr[events_ind];
      title(titlestr[events_ind]);
      ytics(yticmat[events_ind,1],yticmat[events_ind,2],yticmat[events_ind,3],yticmat[events_ind,4]);
      _pline = (1 ~ 6 ~ 0 ~ 0 ~ hmax ~ 0 ~ 0~ 12 ~ 8);
      xy(xgrid, actprob_act[.,events_ind]~actprob_l[.,events_ind]~actprob_h[.,events_ind]);
      nextwind;
      
      events_ind = events_ind + 1;
endo;
endwind;


ret = tkf2eps(_ptek,"3eqevent.eps");
