/* vm_pitmsepredcheck.g
**
*/
new;
closeall;
cls;
library pgraph;

format /mat /on /mb1 /ros 10,4;
outwidth 256;

#include dsge_lh.src
#include dsge_mod.src
#include dsge_para.src
#include dsge_procs.src;
#include dsge_spec.g;
#include pathspec.g;

/* Load summaries from the actual trajectory
*/
trajstr = "traj0";
lpath   = specpath $+ "\\" $+ trajstr;
prun    = "";
lpit    = lpath $+ "\\pitfreq"  $+ prun $+ ".dat";

open fhpit = ^lpit for read;  
PITfreq_act = readr(fhpit,1);
closeall fhpit;

/* Load summaries from the predicted trajectories
*/
ntraj = 500;

PITfreq_sim = zeros(ntraj,cols(PITfreq_act));

trajind = 1;
do until trajind > ntraj;

   trajstr = "traj" $+ ftos(trajind,"%*.*lf",1,0);

   lpath   = specpath $+ "\\" $+ trajstr;
   prun    = "";
   lpit    = lpath $+ "\\pitfreq"  $+ prun $+ ".dat";

   open fhpit = ^lpit for read;
   PITfreq_sim[trajind,.] = readr(fhpit,1);
   closeall fhpit;

   trajind = trajind+1;

endo;

/* sort columns for simulated statistics
*/
colind = 1;
do until colind > cols(PITfreq_sim);
   PITfreq_col = PITfreq_sim[.,colind];
   PITfreq_col = sortc(PITfreq_col,1);
   PITfreq_sim[.,colind] = PITfreq_col;
   colind = colind+1;
endo;

/* construct percentiles for predictive distribution
*/
chkpct_lind = 1+floor(ntraj*chkpct_l);
chkpct_hind = ceil(ntraj*chkpct_h);

PITfreq_l = PITfreq_sim[chkpct_lind,.];
PITfreq_h = PITfreq_sim[chkpct_hind,.];

/* rearrange the PIT information
*/
PITfreq_act = reshape(PITfreq_act,cols(PITfreq_act)/npitbin_,npitbin_)';
PITfreq_l   = reshape(PITfreq_l,cols(PITfreq_l)/npitbin_,npitbin_)';
PITfreq_h   = reshape(PITfreq_h,cols(PITfreq_h)/npitbin_,npitbin_)';


/* Plot histograms for the probability integral transforms. 
*/

disp_h = 1|4|8;
nfcsts        = nv_*(nv_+1);
nhorizon_disp = rows(disp_h);

begwind;
window(nhorizon_disp,nv_,0);

_ptek    = "figure.tkf";
_plwidth = { 10, 10 } ;
_plctrl  = { 0, 0 };
_pltype  = { 1, 1 };
_pcolor  = { 9, 9 };
_pscreen = 1;
_paxht = 0.4;
_pnumht = 0.3;
fonts("microb");

xtics(0,1,0.2,1);
ytics(0,100,20,1);

zgrid = seqa(0,1/npitbin_,npitbin_) + 0.5/npitbin_;
gridmult  = 10;
zgridmult = seqa(0,1/(gridmult*npitbin_),gridmult*npitbin_) + 0.5/(gridmult*npitbin_);

hind    = 1;
windind = 1;
// iterate over forecast horizon
do until hind > nhorizon_disp;

   h = disp_h[hind];
  
   // select h-step forecasts 
   // each block contains 
   // [uncond ~ cond GDP ~ cond pi ~ cond R] .*. [ GDP forec ~ PI forec ~ R forec]
   PITplt_act = PITfreq_act[.,(h-1)*nfcsts+1:h*nfcsts];
   PITplt_l   = PITfreq_l[.,(h-1)*nfcsts+1:h*nfcsts];
   PITplt_h   = PITfreq_h[.,(h-1)*nfcsts+1:h*nfcsts];
   
   varforcind = 1;
   // iterate over variables to be forecast
   // first nv_ forecasts are unconditional
   do until varforcind > nv_;
      
      setwind(windind);

      ylabel(ftos(h,"%*.*lf",1,0) $+ "step");
      xlabel(titlestr[varforcind,.]);

      // generate the plot     
      _pbarwid = 0.98;
      _pline = (1 ~ 6 ~ 0 ~ 100/npitbin_ ~ 1 ~ 100/npitbin_ ~ 0~ 12 ~ 8);
      bar(zgrid,PITplt_act[.,varforcind]);
      xy(zgridmult, (PITplt_l[.,varforcind].*.ones(gridmult,1))
                   ~(PITplt_h[.,varforcind].*.ones(gridmult,1)) );
            
      varforcind = varforcind + 1;
      windind = windind + 1;

   endo;
     
   hind = hind + 1;
endo;
endwind;

ret = tkf2eps(_ptek,"3eqpitunconditional.eps");
