/* vm_pitmsepredcheck.g
**
*/
new;
closeall;
cls;
library pgraph;

format /mat /on /mb1 /ros 10,4;
outwidth 256;

#include dsge_lh.src
#include dsge_mod.src
#include dsge_para.src
#include dsge_procs.src;
#include dsge_spec.g;
#include pathspec.g;


/* Load summaries from the actual trajectory
*/
trajstr = "traj0";
lpath   = specpath $+ "\\" $+ trajstr;
prun    = "";
lmse    = lpath $+ "\\mse"      $+ prun $+ ".dat";

open fhmse = ^lmse for read;
MSE_act     = readr(fhmse,1);
closeall fhmse;

/* Load summaries from the predicted trajectories
*/
ntraj = 500;

MSE_sim     = zeros(ntraj,cols(MSE_act));

trajind = 1;
do until trajind > ntraj;

   trajstr = "traj" $+ ftos(trajind,"%*.*lf",1,0);

   lpath   = specpath $+ "\\" $+ trajstr;
   prun    = "";
   lmse    = lpath $+ "\\mse"      $+ prun $+ ".dat";

   open fhmse = ^lmse for read;

   MSE_sim[trajind,.]     = readr(fhmse,1);

   closeall fhmse;

   trajind = trajind+1;

endo;

/* relative RMSEs
*/
RMSErel_sim = zeros(rows(MSE_sim),nv_^2*hmax);
RMSErel_act = zeros(rows(MSE_act),nv_^2*hmax);

hind = 1;
do until hind > hmax;

   hshift1 = (hind-1)*nv_*nv_;
   hshift2 = (hind-1)*(nv_*(nv_+1));
   condind = 1;
 
   "Horizon" hind;

   sqrt(MSE_act[.,hshift2+1:hshift2+(nv_+1)*nv_]);
   
   do until condind > nv_;
      RMSErel_sim[.,hshift1+(condind-1)*nv_+1:hshift1+condind*nv_]
         = sqrt( MSE_sim[.,hshift2+condind*nv_+1:hshift2+(condind+1)*nv_]
            ./ MSE_sim[.,hshift2+1:hshift2+nv_] );
      RMSErel_act[.,hshift1+(condind-1)*nv_+1:hshift1+condind*nv_]
         = sqrt( MSE_act[.,hshift2+condind*nv_+1:hshift2+(condind+1)*nv_]
            ./ MSE_act[.,hshift2+1:hshift2+nv_] );
      condind = condind+1;
   endo;
   RMSErel_act[.,hshift1+1:hshift1+nv_*nv_];
   "--------------------------------";   

   hind = hind+1;

endo;    

/* sort columns for simulated statistics
*/

colind = 1;
do until colind > cols(RMSErel_sim);
   RMSErel_col = RMSErel_sim[.,colind];
   RMSErel_col = sortc(RMSErel_col,1);
   RMSErel_sim[.,colind] = RMSErel_col;
   colind = colind+1;
endo;

/* construct percentiles for predictive distribution
*/
chkpct_lind = 1+floor(ntraj*chkpct_l);
chkpct_hind = ceil(ntraj*chkpct_h);

RMSErel_l     = RMSErel_sim[chkpct_lind,.];
RMSErel_h     = RMSErel_sim[chkpct_hind,.];

/* Rearrange the MSEs
**   Y|Y pi|Y R|Y - Y|pi pi|pi R|pi - Y|R pi|R R|R
** h1
** h2
*/

RMSErel_l   = reshape(RMSErel_l,hmax,nv_*nv_);
RMSErel_h   = reshape(RMSErel_h,hmax,nv_*nv_);
RMSErel_act = reshape(RMSErel_act,hmax,nv_*nv_);

RMSErelplot_l   = zeros(hmax,(nv_-1)*nv_);
RMSErelplot_h   = zeros(hmax,(nv_-1)*nv_);
RMSErelplot_act = zeros(hmax,(nv_-1)*nv_);

/* Create ylabelstr and eliminate x|x forecasts
*/
plotind = 0;
ylabelstr = " ";

varind = 1;
do until varind > nv_;

   condind = 1;
   do until condind > nv_;

      if varind ne condind;
            plotind = plotind+1;
            RMSErelplot_l[.,plotind]   = RMSErel_l[.,(condind-1)*nv_+varind];
            RMSErelplot_h[.,plotind]   = RMSErel_h[.,(condind-1)*nv_+varind];
            RMSErelplot_act[.,plotind] = RMSErel_act[.,(condind-1)*nv_+varind];
            ylabelstr = ylabelstr $| ( titlestr[varind] $+ " | " $+ titlestr[condind] );
      endif;

      condind = condind+1;

   endo;

   varind = varind+1;

endo;
ylabelstr = ylabelstr[2:rows(ylabelstr)];

RMSErelplot_act;

/* Plot relative RMSE figures
*/

begwind;
window((nv_-0),nv_,0);

_ptek    = "figure.tkf";
_plwidth = { 10, 10, 10 } ;
_psymsiz = { 10, 10, 10 };
_plctrl  = { 1, 1, 1 };
_pltype  = { 6, 1, 1 };
_pstype  = { 1, 2, 2};
_pcolor  = { 12, 9, 9 };
_pscreen = 1;
_paxht = 0.4;
_pnumht = 0.3;
fonts("microb");

xtics(1,8,1,1);
xgrid=seqa(1,1,hmax);
yticmat = (0.5~1.25~0.25~2) |
          (0.5~1.25~0.25~2) |
          (0.5~1.25~0.25~2) ;

varind = 1;
do until varind > nv_;

   condind = 1;
   do until condind > (nv_-1);

      windind   = (condind-1)*nv_+varind;
      columnind = (varind-1)*(nv_-1)+condind;

      windind;
      $ylabelstr[columnind];

      setwind(windind);
      ylabel(ylabelstr[columnind]);
      ytics(yticmat[varind,1],yticmat[varind,2],yticmat[varind,3],yticmat[varind,4]);
      _pline = (1 ~ 6 ~ 0 ~ 1 ~ hmax ~ 1 ~ 0~ 12 ~ 8);
      
      xy(xgrid, RMSErelplot_act[.,columnind]~RMSErelplot_l[.,columnind]~RMSErelplot_h[.,columnind]);
      
      condind = condind + 1;
   endo;
     
   varind = varind + 1;
endo;
endwind;
ret = tkf2eps(_ptek,"3eqrmseratio.eps");
/* ret = tkf2eps(_ptek,odir $+ "pit_cdf_mh" $+ mhrun $+ "p" $+ prun $+".eps");*/

