/* tab_pitchi2predcheck.g
**
*/
new;
closeall;
cls;
library pgraph;

format /mat /on /mb1 /ros 10,4;
outwidth 256;

#include dsge_lh.src
#include dsge_mod.src
#include dsge_para.src
#include dsge_procs.src;
#include dsge_spec.g;
#include pathspec.g;

chkpct_h = 0.9;
use_simulated_p_for_h1 = 1;

/* Load summaries from the actual trajectory
*/
trajstr = "traj0";
lpath   = specpath $+ "\\" $+ trajstr;
prun    = "";
lpitchi2= lpath $+ "\\pitchi2" $+ prun $+ ".dat";

open fhpitchi2 = ^lpitchi2 for read;
PITchi2_act    = readr(fhpitchi2,1);
closeall fhpitchi2;

/* Load summaries from the predicted trajectories
*/
ntraj = 500;

PITchi2_sim = zeros(ntraj,cols(PITchi2_act));

trajind = 1;
do until trajind > ntraj;

   trajstr = "traj" $+ ftos(trajind,"%*.*lf",1,0);

   lpath   = specpath $+ "\\" $+ trajstr;
   prun    = "";
   lpitchi2= lpath $+ "\\pitchi2" $+ prun $+ ".dat";

   open fhpitchi2 = ^lpitchi2 for read;
   PITchi2_sim[trajind,.] = readr(fhpitchi2,1);

   closeall fhpitchi2;

   trajind = trajind+1;

endo;

/* sort columns for simulated statistics
*/

colind = 1;
do until colind > cols(PITchi2_sim);
   PITchi2_col = PITchi2_sim[.,colind];
   PITchi2_col = sortc(PITchi2_col,1);
   PITchi2_sim[.,colind] = PITchi2_col;
   colind = colind+1;
endo;

/* construct percentiles for predictive distribution
*/
chkpct_hind = ceil(ntraj*chkpct_h);
PITchi2_h   = PITchi2_sim[chkpct_hind,.];

/* Construct p values using the predictive distribution
*/
PITchi2_p = zeros(1, cols(PITchi2_h));
npitbin = 5;
PITbin  = seqa(1/npitbin,1/npitbin,npitbin);

j = 1;
s_mat = zeros(rows(PITchi2_sim), 1);
nsamp = 46;
do until j > rows(PITchi2_sim);//(nv_+1)*nv_;
   pits = rndu(nsamp, 1);
   PITfreq = counts(pits, PITbin);
   s_mat[j] = sumc( (PITfreq - nsamp/npitbin)^2 )/(nsamp/npitbin);

   j = j + 1;
endo;

do until j > (nv_+1)*nv_;
   PITchi2_sim[., j] = s_mat;
   j = j + 1;
endo;

i = 1;
do until i > cols(PITchi2_h);
   PITchi2_p[., i] = sumc(PITchi2_act[., i] .< PITchi2_sim[., i]) / ntraj;
   i = i + 1;
endo;

/* Rearrange the chi2stats
**   Y pi R - Y|Y pi|Y R|Y - Y|pi pi|pi R|pi - Y|R pi|R R|R
** h1
** h2
*/
PITchi2_h   = reshape(PITchi2_h,hmax,nv_*(nv_+1));
PITchi2_act = reshape(PITchi2_act,hmax,nv_*(nv_+1));
PITchi2_p   = reshape(PITchi2_p,hmax,nv_*(nv_+1));
/* display a subset of the results
*/
PITchi2_h   = PITchi2_h[1 4 8,.];
PITchi2_act = PITchi2_act[1 4 8,.];
PITchi2_p   = PITchi2_p[1 4 8, .];

/* Conditioning str
*/
condstr = "Uncond.        " $|
          "Cond on $Y$    " $|
          "Cond on $\\pi$  " $|
          "Cond on $R$    ";

PITchi2_crit = PITchi2_p;
hind = 1;
do until hind > rows(PITchi2_h);

   condind = 1;
   do until condind > nv_+1;
   
      tablerow = condstr[condind];
      
      varind = 1;
      do until varind > nv_;
     
         if (condind > 1) AND ( (condind-1)==varind);
            tablerow = tablerow $+ " &       &      ";
         else;
            tablerow = tablerow 
                  $+ " & " $+ ftos(PITchi2_act[hind,(condind-1)*nv_+varind],"%*.*lf",5,2)
                  $+ " & ("  
                  $+ ftos(PITchi2_crit[hind,(condind-1)*nv_+varind],"%*.*lf",5,2)
                  $+ " ) ";
         endif;
         varind = varind + 1;
        
      endo;
      
      tablerow = tablerow $+ "\\\\";
      $tablerow;
      
      condind = condind+1;
      
   endo;
  
   "----------------";
   
   hind = hind+1;
endo;



