/* vm_pitmsepredcheck.g
**
*/
new;
closeall;
cls;

format /mat /on /mb1 /ros 10,4;
outwidth 256;

#include dsge_lh.src
#include dsge_mod.src
#include dsge_para.src
#include dsge_procs.src;
#include dsge_spec.g;
#include pathspec.g;


/* Load summaries from the actual trajectory
*/
trajstr = "traj0";
lpath   = specpath $+ "\\" $+ trajstr;
prun    = "";
lpit    = lpath $+ "\\pitfreq"  $+ prun $+ ".dat";
lmse    = lpath $+ "\\mse"      $+ prun $+ ".dat";

open fhpit = ^lpit for read;
open fhmse = ^lmse for read;
   
PITfreq_act = readr(fhpit,1);
MSE_act     = readr(fhmse,1);

closeall fhpit, fhmse;

/* Load summaries from the predicted trajectories
*/
ntraj = 5;

PITfreq_sim = zeros(ntraj,cols(PITfreq_act));
MSE_sim     = zeros(ntraj,cols(MSE_act));

trajind = 1;
do until trajind > ntraj;

   trajstr = "traj" $+ ftos(trajind,"%*.*lf",1,0);

   lpath   = specpath $+ "\\" $+ trajstr;
   prun    = "";
   lpit    = lpath $+ "\\pitfreq"  $+ prun $+ ".dat";
   lmse    = lpath $+ "\\mse"      $+ prun $+ ".dat";

   open fhpit = ^lpit for read;
   open fhmse = ^lmse for read;
   
   PITfreq_sim[trajind,.] = readr(fhpit,1);
   MSE_sim[trajind,.]     = readr(fhmse,1);

   closeall fhpit, fhmse;

   trajind = trajind+1;

endo;

/* sort columns for simulated statistics
*/
colind = 1;
do until colind > cols(PITfreq_sim);
   PITfreq_col = PITfreq_sim[.,colind];
   PITfreq_col = sortc(PITfreq_col,1);
   PITfreq_sim[.,colind] = PITfreq_col;
   colind = colind+1;
endo;

colind = 1;
do until colind > cols(MSE_sim);
   MSE_col = MSE_sim[.,colind];
   MSE_col = sortc(MSE_col,1);
   MSE_sim[.,colind] = MSE_col;
   colind = colind+1;
endo;

/* construct percentiles for predictive distribution
*/
chkpct_lind = 1+floor(ntraj*chkpct_l);
chkpct_hind = ceil(ntraj*chkpct_h);

PITfreq_l = PITfreq_sim[chkpct_lind,.];
PITfreq_h = PITfreq_sim[chkpct_hind,.];
MSE_l     = MSE_sim[chkpct_lind,.];
MSE_h     = MSE_sim[chkpct_hind,.];

/* rearrange the PIT information
*/
PITfreq_act = reshape(PITfreq_act,cols(MSE_act),npitbin_)';
PITfreq_l   = reshape(PITfreq_l,cols(MSE_act),npitbin_)';
PITfreq_h   = reshape(PITfreq_h,cols(MSE_act),npitbin_)';

/* Print the results
*/
disp_h = 1|2|4;
nfcsts        = nv_*(nv_+1);
nhorizon_disp = rows(disp_h);

"--------------------------------------------------------------";
" Root mean squared errors";
"--------------------------------------------------------------";
horizonind = 1;
do until horizonind > nhorizon_disp;

   h = disp_h[horizonind];
   h;; "-step ahead forecast";
   MSEh = MSE_act[.,(h-1)*nfcsts+1:h*nfcsts];
   
   z = seqa(1,nv_,nv_+1);
   j = 1;
   do until j > nv_;
      MSEjh  = MSEh[.,z];
      RMSEjh = sqrt(MSEjh);
      titlestr[j,.];; RMSEjh;
      z = z+1;
      j = j+1;
   endo;
   "";
   horizonind = horizonind + 1;
endo;


"--------------------------------------------------------------";
" Properties of the PITs";
"--------------------------------------------------------------";

horizonind = 1;
do until horizonind > nhorizon_disp;
   
   h = disp_h[horizonind];
   h;; "-step ahead forecast";
   
   "                    Frequencies                       ";

   PITh = PITfreq_act[.,(h-1)*nfcsts+1:h*nfcsts];
   
   z = seqa(1,nv_,nv_+1);

   j = 1;
   do until j > nv_;
      titlestr[j,.];; PITh[.,z];
      
      z = z+1;
      j = j+1;
	 
   endo;
   "";

   horizonind = horizonind + 1;
	 
endo;

end;